import time


class Time(object):
    def __init__(self):
        self.fmt = "%Y-%m-%d %H:%M:%S"
        self.time = time.time()

    def set_fmt(self, fmt):
        self.fmt = fmt

    @property
    def get_fmt_time(self):
        now = int(time.time())
        time_array = time.localtime(now)
        other_style_time = time.strftime(self.fmt, time_array)
        return other_style_time



import time
import threading
from tools.logger import Logger


class Queue(object):
    def __init__(self):
        self._queue = []
        self.logger = Logger()

    @property
    def queue(self):
        return self._queue

    def put(self, item):
        self.queue.append(item)

    def get(self):
        if not self.empty():
            return self.queue.pop(0)
        else:
            self.logger.warning('Queue is empty!')

    def empty(self):
        length = len(self.queue)
        if length == 0:
            return True
        else:
            return False

    @property
    def length(self):
        return len(self.queue)

    def __repr__(self):
        return '{}|{}'.format(self.length, self.queue)



from tools.timer import Time
import traceback
import sys
import os
from threading import Lock, get_ident, current_thread
from functools import wraps


# import inspect

# 获取被调用函数所在模块文件名
# print(inspect.stack()[1][1])
# print(sys._getframe(1).f_code.co_filename)

# 获取被调用函数名称
# print(inspect.stack()[1][3])
# print(sys._getframe(1).f_code.co_name)

# 获取被调用函数在被调用时所处代码行数
# print(inspect.stack()[1][2])
# print(sys._getframe(1).f_lineno)


class Logger(object):
    def __init__(self, level=0, full='None'):
        self.time = Time()
        self.time.set_fmt('%H:%M:%S')
        self.level = level
        self.full = full
        self.attri_list = None
        self.mode = None
        self.pre_module = None
        self.pre_func = None
        self.line_num = None
        self.fmt = None
        self.log_dict = None

    def info(self, msg):
        level = 1
        if level >= self.level:
            mode = self.mode[level - 1]
            print(self.fmt.format(mode, msg))

    def warning(self, msg):
        level = 2
        if level >= self.level:
            mode = self.mode[level - 1]
            print(self.fmt.format(mode, msg))

    def error(self, msg=None):
        level = 3
        if level >= self.level:
            mode = self.mode[level - 1]
            type_, value_, traceback_ = sys.exc_info()
            ex = traceback.format_exception(type_, value_, traceback_)
            print(self.fmt.format(mode, '\033[1;31m{}\033[0m'.format('*' * 100)))
            for i in ex[1:]:
                print(self.fmt.format(mode, i.splitlines(True)[0].strip()))

            if msg:
                print(self.fmt.format(mode, msg))
            print(self.fmt.format(mode, '\033[1;31m{}\033[0m'.format('*' * 100)))

    def log(self, level=0, attri_list=['time', 'thread', 'func'], full=False):
        def L(func):
            @wraps(func)
            def wrapper(*args, **kwargs):
                self.mode = ['\033[1;32mINFO\033[0m', '\033[1;33mWARNING\033[0m', '\033[1;31mERROR\033[0m']
                self.time = Time()
                self.time.set_fmt('%H:%M:%S')
                self.pre_module = sys._getframe(1).f_code.co_filename.split('/')[-1].split('.')[0]
                self.pre_func = func.__qualname__
                self.line_num = sys._getframe(1).f_lineno
                self.fmt = '\r'
                self.log_dict = None
                self.log_dict = {
                    'time': '[{}] '.format(self.time.get_fmt_time),
                    'thread': '[{}] '.format(current_thread().name),
                    'module': '[{}] '.format(self.pre_module),
                    'func': '[{}] '.format(self.pre_func),
                    'num': '[{}] '.format(self.line_num),
                    'index': '[{}|{} {}] '.format(self.pre_module, self.pre_func, self.line_num)
                }
                Full = full
                if self.full != 'None':
                    Full = self.full
                if Full:
                    self.attri_list = ['time', 'thread', 'index']
                else:
                    self.attri_list = attri_list
                for i in self.attri_list:
                    self.fmt = self.fmt + self.log_dict[i]
                self.fmt = self.fmt + '[{}]: {}'
                return func(*args, **kwargs)

            return wrapper

        return L


def __call__(self, *args, **kwargs):
    self.info(args, kwargs)



from openpyxl import Workbook
from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker
from tools.logger import Logger
from contextlib import contextmanager


class Sql(object):

    def __init__(self, user, password, port, dbname, char, base):
        self.engine = create_engine(
            'mysql://{}:{}@localhost:{}/{}?charset={}'.format(user, password, port, dbname, char),
            echo=False)
        self.__Base = base
        self.__Base.metadata.create_all(self.engine)
        self.__Session = sessionmaker(bind=self.engine)
        self.__session = self.__Session()

    def write(self, msg):
        self.__session.add(msg)
        self.__session.commit()

    @property
    def session(self):
        return self.__session


class Excel(object):
    def __init__(self, path):
        self.wb = Workbook()
        self.sheet = self.wb.active
        self.path = path + '.xlsx'

    def write_headline(self, head_tag_list, head_title_list):
        head_tag_list = list(head_tag_list)
        for i, h in enumerate(head_title_list):
            self.sheet[head_tag_list[i] + '1'] = h
        self.wb.save(self.path)

    def save(self):
        self.wb.save(self.path)


class Csv(object):

    def __init__(self, head_line, save_path):
        self.lg = Logger()
        cls = self
        self.head_line = head_line
        self.save_path = save_path
        self.write_headline()

    def write_headline(self):
        with open(self.save_path, 'a+') as f:
            line = [str(i) for i in self.head_line]
            f.write(','.join(line) + '\n')

    @contextmanager
    def write_body(self, file, *args):
        for i in range(len(args[1])):
            line = []
            for j in range(len(args)):
                line.append(args[j][i])
            line = ','.join([str(i) for i in line]) + '\n'
            file.write(line)
            file.flush()
        yield file
        self.lg.info('Write Done!')



from threading import Thread
import time
import threading
from tools.logger import Logger
from concurrent.futures.thread import ThreadPoolExecutor
from tools.queue import Queue


class RUNThread(object):
    def __init__(self, thread):
        self.thread = thread
        self.run()

    def run(self):
        if isinstance(self.thread, list):
            for thread in self.thread:
                thread.start()
            for thread in self.thread:
                thread.join()
        elif isinstance(self.thread, (Thread, MYThread)):
            self.thread.start()
            self.thread.join()


class ThreadPool(object):
    def __init__(self, max_workers, info=True):
        self.max_workers = max_workers
        self._task_queue = Queue()
        self.waiting_list = []
        self.working_list = []
        self.logger = Logger()
        self.info = info
        self.stop = False

    @property
    def task_queue(self):
        return self._task_queue

    def add_task(self, func, *args, **kwargs):
        if isinstance(args[0], tuple):
            task = (func, args[0], kwargs)
        else:
            task = (func, args, kwargs)
        self.task_queue.put(task)

    def add_task_list(self, func, arg_list):
        for i in arg_list:
            self.add_task(func, i)

    def wait_work_done(self):
        for i in self.waiting_list:
            i.start()
            self.waiting_list.remove(i)
            self.working_list.append(i)
        for i in self.working_list:
            if not i.is_alive():
                i.join()
                if self.info:
                    self.logger.info(
                        '-' * 30 + ' Queue Length is {}, {} workers is working '.format(self.task_queue.length,
                                                                                        len(
                                                                                            self.working_list)) + '-' * 30)
                self.working_list.remove(i)

    def run(self):
        if self.task_queue.length < self.max_workers:
            self.max_workers = self.task_queue.length
        if self.info:
            self.logger.info(
                '-' * 30 + ' Queue Length is {}, {} workers is working '.format(self.task_queue.length,
                                                                                self.max_workers) + '-' * 30)
        while not self.stop:
            time.sleep(1)
            while len(self.waiting_list) + len(
                    self.working_list) < self.max_workers and self.task_queue.length != 0:
                func, args, kwargs = self.task_queue.get()
                worker = Thread(target=func, args=args, kwargs=kwargs)
                self.waiting_list.append(worker)
                self.wait_work_done()
            if len(self.working_list) != 0:
                self.wait_work_done()
            else:
                self.stop = True

